import { BatchWriteItemCommand, DeleteItemCommand, PutItemCommand, UpdateItemCommand } from '@aws-sdk/client-dynamodb';
import { unmarshall } from '@aws-sdk/util-dynamodb';
import { DDBReadOnlyClient } from './DDBReadOnlyClient';
import { mapUpdateItemCommand } from './mapper/mapUpdateItemCommand';
import { mapDeleteItemsCommand } from './mapper/mapDeleteItemsCommand';
import { mapDeleteItemCommand } from './mapper/mapDeleteItemCommand';
import { mapCreateItemsCommand } from './mapper/mapCreateItemsCommand';
import { mapCreateItemCommand } from './mapper/mapCreateItemCommand';
import { formatKey } from './formatter/formatKey';
export class DDBClient extends DDBReadOnlyClient {
    constructor(configuration) {
        super(configuration);
    }
    async createItem(item) {
        try {
            const partitionKey = item[this.config.partitionKeyName];
            const sortKey = this.config.sortKeyName ? item[this.config.sortKeyName] : undefined;
            this.requireFullPrimaryKey(partitionKey, sortKey);
            await this.client.send(new PutItemCommand(mapCreateItemCommand(this.config, item)));
            return item;
        }
        catch (err) {
            console.error('item', JSON.stringify(item));
            throw new Error(err);
        }
    }
    async createItems(items) {
        try {
            await this.client.send(new BatchWriteItemCommand(mapCreateItemsCommand(this.config, items)));
        }
        catch (err) {
            console.error('items', JSON.stringify(items));
            throw new Error(err);
        }
    }
    async updateItem(item, createIfNotExists = false) {
        const partitionKey = item[this.config.partitionKeyName];
        const sortKey = this.config.sortKeyName && item[this.config.sortKeyName];
        try {
            this.requireFullPrimaryKey(partitionKey, sortKey);
            const response = await this.client.send(new UpdateItemCommand(mapUpdateItemCommand(this.config, item, createIfNotExists)));
            if (!response.Attributes)
                throw new Error('Updated item not returned');
            return unmarshall(response.Attributes);
        }
        catch (error) {
            if (error.name === 'ConditionalCheckFailedException') {
                throw new Error(`Can't update item as it does not exist: ${formatKey(this.config, partitionKey, sortKey)}`);
            }
            console.error('item', JSON.stringify(item));
            throw new Error(error);
        }
    }
    async deleteItem(partitionKey, sortKey) {
        try {
            this.requireFullPrimaryKey(partitionKey, sortKey);
            await this.client.send(new DeleteItemCommand(mapDeleteItemCommand(this.config, partitionKey, sortKey)));
        }
        catch (err) {
            console.error(formatKey(this.config, partitionKey, sortKey));
            throw new Error(err);
        }
    }
    async deleteItems(items) {
        try {
            if (items.length === 0)
                return;
            await this.client.send(new BatchWriteItemCommand(mapDeleteItemsCommand(this.config, items)));
        }
        catch (err) {
            console.error('items', JSON.stringify(items));
            throw new Error(err);
        }
    }
}
